//#include "..\..\..\Common\InstanceNoodles.fxh"

StructuredBuffer<float3> PosIn;
RWBuffer<float4> PrevBuffer; //Sorting Buffer from previous frame;
float3 vector1Value;
RWStructuredBuffer<float> RWValueBuffer : BACKBUFFER;

float threadCount;


//Odd-EvenMergeSort
float3 camPos <string uiname="Camera Position";>;
bool reset <string uiname="Reset";>;

uniform float3 Param1;
uniform float3 Param2;

// contents of the uniform data fields
#define GroupSize Param1.x    // Sub-groups size (2,4,4,8,8,8,16,16,16,16, ...
#define Width Param1.y        // x resolution
#define Height Param1.z       // y resolution

#define distPartner Param2.x  // distance of the partner to compare
#define offsetDown Param2.y   // inferior bound from where active cells start
#define offsetUp Param2.z     // superior bound where active cells end


//Instance Noodle adoption 
uint bSize (StructuredBuffer<float3> buffer)
{
	uint count, dummy;	
	buffer.GetDimensions(count,dummy);
	return count;
}

float3 bLoad(StructuredBuffer<float3> valueBuffer, float3 defaultValue, uint dtid)
{
	float3 value = defaultValue;
	uint count = bSize(valueBuffer);
	if (count > 0) value = valueBuffer[dtid.x % count];
	return value;
}

[numthreads(64,1,1)]
void CS_Distance (uint3 iD : SV_DispatchThreadID)
{
	//if (i.x > threadCount) { return; }
	uint count1, dummy;	
	PosIn.GetDimensions(count1,dummy);
	
	float3 positions = bLoad(PosIn, vector1Value, iD.x);
	
	////////////////////SORT////////////////////
	
	// get the index in the array
  float i = iD.x;//floor(In.TexCd.x * Width) + floor(In.TexCd.y * Height) * Width;
 
  // get self data from the last frame texture
  float4 self = PrevBuffer[iD.x];  
  
  // my position within the sub-group to merge
  float j = floor(fmod(i, GroupSize));
  
  float compare;
  float direction; 
  float4 SortedValue;

  // find if the pixel is in the active interval of the sub-group
  bool active = (j >= offsetDown) && (j <= offsetUp);
  
  if (!active) //inactive
     {
      // must copy -> compare with self
      compare = 0.0;
      SortedValue = self;
     }
    
  else
     {
      // must sort

      // find the direction where to find the partner (1 = right , 0 = left)
      direction = fmod(((j + offsetDown) / distPartner) , 2) >= 1;
          
            if (direction)  // we are on the right side -> compare with partner on the left
               compare = -1.0;

               else         // we are on the left side -> compare with partner on the right
               compare = 1.0;

      // get the partner index in the array
      float adr = i + (compare * distPartner);
      // convert the array index in UV coordinates and get the partner's data from data texture
      float4 partner = PrevBuffer[adr];
     
      // compare vales; on left is < , on right is >=
      if (direction) { SortedValue = (self.x >=  partner.x) ? partner : self; }
      else           { SortedValue = (self.x <   partner.x) ? partner : self; }
     }



  
  if(reset) { SortedValue.y = i; }  //array index reset on Reset
  
  // Update the sorted Key value (distance from Camera)
  
  // Get Position from incoming position texture using the index from the sorted values
  float3 getPos = PosIn[SortedValue.y];
  // evaluate the new distance from Camera
  //float newDist = length(camPos - getPos);  
  float newDist = distance(getPos, camPos);
  // update the key value
  SortedValue.x = newDist;
  
  PrevBuffer[iD.x] = SortedValue.x;   //RWBuffer as feedback???

	
  RWValueBuffer[iD.x] = PrevBuffer[iD.x].x;	
}





technique11 Distance
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CS_Distance() ) );
	}
}

